#!/usr/bin/perl -w

use strict;

my %opt_is_race_array;  # value is pretty-printed name
my %opt_value;
my $ofmt = "-";

if (!@ARGV) {
    print "usage: $0 [def=config.hi4] [src=pconfig.src] [ofmt=output-file-template]

def=config.hi4: name of file containing option definitions (config.hi4)

src=pconfig.src: file name to be read as pconfig.src (option values)

ofmt=output-file-template: template can contain one '\%d', and will cause output
   to be written to 11 files (default: 11x stdout)\n";
    exit 1;
}

foreach (@ARGV) {
    if (m{^src=(.*)}) {
        my $reading = 1;
        open FILE, "< $1" or die "$1: $!";
        while (<FILE>) {
            chomp;
            if (/^\s*\#/) {
                # nix
            } elsif (/^\s*%\s*phost/i) {
                $reading = 1;
            } elsif (/^\s*%/) {
                $reading = 0;
            } elsif (/^\s*(\S+)\s*=\s*([^\#]*)/ && $reading) {
                $opt_value{lc($1)} = $2;
            }
        }
        close FILE;
    } elsif (m{^ofmt=(.*)}) {
        $ofmt = $1;
    } elsif (m{^def=(.*)}) {
        open FILE, "< $1" or die "$1: $!";
        while (<FILE>) {
            if (/CFDefine4\(([a-z0-9]+).*"(.*)"\)/i) {
                my ($name, $value) = ($1, $2);
                if (/RACEARRAY/) {
                    $opt_is_race_array{lc($name)} = $name;
                    $opt_value{lc($name)} = $value;
                }
            }
        }
        close FILE;
    } else {
        die "invalid parameter: '$_'\n";
    }
}

# we have now read everything.
foreach (1 .. 11) {
    open FILE, ">".sprintf($ofmt,$_) or die "$!";
    print FILE "# Race definition for player $_\n\n";
    foreach my $k (sort keys %opt_is_race_array) {
        my @value = split /,\s*/, $opt_value{$k};
        while (@value < 11) { push @value, $value[-1]; }

        my $value = $value[$_-1];
        my $diffs = 0;
        my $deviator;
        my $comment;
        foreach my $c (1 .. 11) {
            if ($c != $_ && $value[$c-1] ne $value) {
                ++$diffs;
                $deviator = $c;
            }
        }

        if ($diffs == 0) {
            # no differences
            $comment = "everyone has this value";
        } elsif ($diffs == 1) {
            # one difference
            $comment = "player $deviator has `$value[$deviator-1]' here";
        } elsif ($diffs == 10) {
            my $alleq = 1;
            foreach my $c (1 .. 11) {
                if ($c != $_ && $value[$c-1] ne $value[$deviator-1]) {
                    $alleq = 0;
                }
            }
            if ($alleq) {
                $comment = "everyone else has `$value[$deviator-1]' here";
            }
        }

        if (defined($comment)) {
            printf FILE "%-30s = %-10s  # %s\n", $opt_is_race_array{$k}, $value, $comment;
        } else {
            printf FILE "%-30s = %s\n", $opt_is_race_array{$k}, $value;
        }
    }
    close FILE;
}
